"""
A class to interact with Marketplace's api, using OAuth.

For full spec please read Marketplace API documentation
https://github.com/mozilla/zamboni/blob/master/docs/topics/api.rst
"""

import json
import logging
import mimetypes

from base64 import b64encode

import oauth2 as oauth

from urlparse import urlunparse

from .connection import Connection

log = logging.getLogger('marketplace.%s' % __name__)

MARKETPLACE_PORT = 443
MARKETPLACE_DOMAIN = 'marketplace.mozilla.org'
MARKETPLACE_PROTOCOL = 'https'

URLS = {'validate': '/apps/validation/',
        'validation_result': '/apps/validation/%s/',
        'create': '/apps/app/',
        'app': '/apps/app/%s/',
        'create_screenshot': '/apps/preview/?app=%s',
        'screenshot': '/apps/preview/%s/',
        'categories': '/apps/category/',
        'enable': '/apps/status/%s/'}


class Client:
    """A base class to authenticate and work with Marketplace OAuth.
    """
    signature_method = oauth.SignatureMethod_HMAC_SHA1()

    def __init__(self, domain=MARKETPLACE_DOMAIN,
                 protocol=MARKETPLACE_PROTOCOL,
                 port=MARKETPLACE_PORT,
                 prefix='',
                 consumer_key=None, consumer_secret=None):
        self.domain = domain
        self.protocol = protocol
        self.port = port
        self.prefix = prefix
        self.conn = None
        if consumer_secret and consumer_key:
            self.conn = self.get_connection(consumer_key, consumer_secret)

    @staticmethod
    def get_connection(consumer_key, consumer_secret):
        """Provide Connection object used for communication with the API
        """
        return Connection(consumer_key, consumer_secret)

    def url(self, key):
        """Creates a full URL to the API using urls dict
        """
        return urlunparse((self.protocol, '%s:%s' % (self.domain, self.port),
                           '%s/api%s' % (self.prefix, URLS[key]),
                           '', '', ''))

    def validate_manifest(self, manifest_url):
        """Order manifest validation

        :returns: dict with an ``id`` to check the result
        """
        # there is a bug request to make this synchronous on Marketplace side
        # this will return the same as :method:`get_manifest_validation_result`
        return self.conn.fetch('POST',
                               self.url('validate'),
                               {'manifest': manifest_url})

    def get_manifest_validation_result(self, manifest_id):
        """Check if the manifest is processed and if it's valid

        :param: manifest_id (string) id received in :method:`validate_manifest`
        :returns: (HttpResponse)
            * status_code - 200 if manifest in validation
            * content - (dict) with some important fields alongs the other:
                * processed (Boolean) has manifest been processed?
                * valid (Boolean) is manifest valid?
                * validation - empty string if valid else error dict
        """
        return self.conn.fetch('GET',
                               self.url('validation_result') % manifest_id)

    def is_manifest_valid(self, manifest_id):
        """Check validation shortcut

        :param: manifest_id (string) id received in :method:`validate_manifest`
        :returns:
            * True if manifest was valid
            * None if manifest wasn't checked yet
            * validation dict if not valid
        """
        response = self.get_manifest_validation_result(manifest_id)
        if response.status_code != 200:
            raise Exception(response.status_code)
        content = json.loads(response.content)
        if not content['processed']:
            return None
        if content['valid']:
            return True
        return content['validation']

    def create(self, manifest_id):
        """Issue create process

        :returns: HttpResponse:
            * status_code - 201 if successful
            * content - dict with some important fields:
                * id (string) application id in marketplace
                * resource_uri (string) url in marketplace
                * slug (string) unique name in marketplace
        """
        return self.conn.fetch('POST', self.url('create'),
                               {'manifest': '%s' % manifest_id})

    def list_webapps(self):
        """Lists all webapps owned by user

        :returns: list
        """
        response = self.conn.fetch('GET', self.url('create'))
        return response

    def update(self, app_id, data):
        """Update app identified by app_id with data

        :params:
            * app_id (int) id in the marketplace received with :method:`create`
            * data (dict) some keys are required:
                * *name*: the title of the app. Maximum length 127
                  characters.
                * *summary*: the summary of the app. Maximum length
                  255 characters.
                * *categories*: a list of the categories, at least
                  two of the category ids provided from the category api
                  (see below).
                * *support_email*: the email address for support.
                * *device_types*: a list of the device types at least
                  one of: 'desktop', 'phone', 'tablet'.
                * *payment_type*: only choice at this time is 'free'.
        :returns: HttResponse:
            * status_code (int) 202 if successful
            * content (dict) or empty if successful
        """
        assert ('name' in data
                and data['name']
                and 'summary' in data
                and 'categories' in data
                and data['categories']
                and 'support_email' in data
                and data['support_email']
                and 'device_types' in data
                and data['device_types']
                and 'payment_type' in data
                and data['payment_type']
                and 'privacy_policy' in data
                and data['privacy_policy'])
        return self.conn.fetch('PUT', self.url('app') % app_id, data)

    def status(self, app_id):
        """View details of an app identified by its id

        :returns: HttResponse:
            * status_code (int) 200 if successful
            * content (JSON String) with all available app information
        """
        return self.conn.fetch('GET', self.url('app') % app_id)

    def delete(self, app_id):
        """Delete an app from Marketplace
        """
        # XXX: This isn't yet implemented on API
        # return self.conn.fetch('DELETE', self.url('app') % app_id)
        raise NotImplementedError()

    def create_screenshot(self, app_id, filename, position=1):
        """Add a screenshot to the web app identified by by ``app_id``.
        Screenshots are ordered by ``position``.

        :returns: HttpResponse:
            * status_code (int) 201 is successful
            * content (dict) containing screenshot data
        """
        # prepare file for upload
        with open(filename, 'rb') as s_file:
            s_content = s_file.read()
        s_encoded = b64encode(s_content)
        url = self.url('create_screenshot') % app_id

        mtype, encoding = mimetypes.guess_type(filename)
        if mtype is None:
            mtype = 'image/jpeg'

        data = {'position': position,
                'file': {'type': mtype,
                         'data': s_encoded}}
        return self.conn.fetch('POST', url, data)

    def get_screenshot(self, screenshot_id):
        """Get information about screenshot or video

        :returns HttpResponse:
            * status_code (int) 200 is successful
            * content (JSON string)
        """
        return self.conn.fetch('GET', self.url('screenshot') % screenshot_id)

    def del_screenshot(self, screenshot_id):
        """Deletes screenshot

        :returns: HttpResponse:
            * status_code (int) 204 if successful
        """
        return self.conn.fetch('DELETE',
                               self.url('screenshot') % screenshot_id)

    def get_categories(self):
        """Get all categories from Marketplae
        """
        return self.conn.fetch('GET', self.url('categories'))

    def app_state(self, app_id, status=None, disabled_by_user=None):
        """Once all the data has been completed and at least one screenshot created,
        you can push the app to the review queue
        status (optional): key statuses are

            incomplete: incomplete
            pending: pending
            public: public
            waiting: waiting to be public

        disabled_by_user (optional): True or False

        Valid transitions that users can initiate are:

            waiting to be public to public: occurs when the app has been reviewed, but not yet been made public.

            incomplete to pending: call this once your app has been completed and it will be added to the Marketplace review queue.
                This can only be called if all the required data is there. If not, you'll get an error containing the reason
            disabled_by_user: by changing this value from True to False you can enable or disable an app
        """
        assert status is not None or disabled_by_user is not None
        data = {}
        if status:
            data['status'] = status
        if disabled_by_user:
            data['disabled_by_user'] = disabled_by_user

        return self.conn.fetch('PATCH',
                self.url('enable') % app_id, data)



"""Marketplace Client Library

https://wiki.mozilla.org/Marketplace

Mozilla is building a Marketplace to bring personalized discovery,
worldwide distribution, and easy payments to the largest platform
for app development: the Web.

This library helps to create Python based sites or apps to communicate
with Marketplace.
"""

__version__ = '0.1'
__all__ = [
        'Client',
]
__author__ = 'Piotr Zalewa <zalun@mozilla.com>'

from .client import Client



""" Provide connection with Marketplace API
"""

import json
import logging
import time
import urllib

import oauth2 as oauth
import requests

log = logging.getLogger('marketplace.%s' % __name__)


class NotExpectedStatusCode(requests.exceptions.HTTPError):
    """ Raise if status code returned from API is not the expected one
    """
    pass


def _get_args(consumer):
    """Provide a dict with oauth data
    """
    return dict(
        oauth_consumer_key=consumer.key,
        oauth_nonce=oauth.generate_nonce(),
        oauth_signature_method='HMAC-SHA1',
        oauth_timestamp=int(time.time()),
        oauth_version='1.0')


class Connection:
    """ Keeps the consumer class and provides the way to connect to the
    Marketplace API
    """
    signature_method = oauth.SignatureMethod_HMAC_SHA1()
    consumer = None

    def __init__(self, consumer_key, consumer_secret):
        self.set_consumer(consumer_key, consumer_secret)

    def set_consumer(self, consumer_key, consumer_secret):
        """Sets the consumer attribute
        """
        self.consumer = oauth.Consumer(consumer_key, consumer_secret)

    def prepare_request(self, method, url, body=''):
        """Adds consumer and signs the request

        :returns: headers of the signed request
        """
        req = oauth.Request(method=method, url=url,
                            parameters=_get_args(self.consumer))
        req.sign_request(self.signature_method, self.consumer, None)

        headers = req.to_header()
        headers['Content-type'] = 'application/json'
        if body:
            if method == 'GET':
                body = urllib.urlencode(body)
            else:
                body = json.dumps(body)
        return {"headers": headers, "data": body}

    @staticmethod
    def _get_error_reason(response):
        """Extract error reason from the response. It might be either
        the 'reason' or the entire response
        """
        body = response.json
        if body and 'reason' in body:
            return body['reason']
        return response.content

    def fetch(self, method, url, data=None, expected_status_code=None):
        """Prepare the headers, encode data, call API and provide
        data it returns
        """
        kwargs = self.prepare_request(method, url, data)
        response = getattr(requests, method.lower())(url, **kwargs)
        log.debug(str(response.__dict__))
        if response.status_code >= 400:
            response.raise_for_status()
        if (expected_status_code
                and response.status_code != expected_status_code):
            raise NotExpectedStatusCode(self._get_error_reason(response))
        return response

    def fetch_json(self, method, url, data=None, expected_status_code=None):
        """Return json decoded data from fetch
        """
        return self.fetch(method, url, data, expected_status_code).json()



from setuptools import setup

setup(
    name='Marketplace',
    version='0.1.5a',
    packages=['marketplace', ],
    license='Mozilla Public License (MPL 2.0)',
    author='Piotr Zalewa',
    author_email='zalun@mozilla.com',
    url='https://github.com/mozilla/Marketplace.Python',
    long_description="""
Marketplace Client Library

Mozilla is building a Marketplace to bring personalized discovery,
worldwide distribution, and easy payments to the largest platform
for app development: the Web.

This library helps to create Python based sites or apps to communicate
with Marketplace.

You may find the source code and collaborate your time and experience at
https://github.com/mozilla/Marketplace.Python
""",
    install_requires=['httplib2', 'oauth2', 'requests'])
